// src/components/SignatureCanvas.js
import { useRef } from 'react';
import SignatureCanvas from 'react-signature-canvas';
import { Button, message } from 'antd';
import { useTranslation } from '../../../lib';

const SignatureCanvasComponent = ({ onClear, onSave }) => {
    const sigCanvas = useRef({});
    const { t, i18n } = useTranslation(); // To Translate to another language

    const clear = () => {
        sigCanvas.current.clear();
        if (onClear) onClear();
    };

    const save = () => {
        if (onSave) {
            onSave(sigCanvas.current.toDataURL('image/png'));
            message.success(t('Signature Saved'));
        }
    };

    return (
        <div>
            <SignatureCanvas
                ref={sigCanvas}
                canvasProps={{
                    width: 600,
                    height: 200,
                    className: 'sigCanvas',
                    style: { border: '1px solid black' }
                }}
            />
            <div style={{ display: 'flex', justifyContent: 'flex-end', gap: '10px', marginTop: '10px' }}>
                <Button
                    style={{ borderRadius: '2px' }}
                    type="primary"
                    size="medium"
                    onClick={clear}
                >
                    {t('Clear')}
                </Button>

                <Button
                    style={{ borderRadius: '2px' }}
                    type="primary"
                    size="medium"
                    onClick={save}
                >
                    {t('Save')}
                </Button>
            </div>
        </div>

    );
};

export default SignatureCanvasComponent;
